/*
 * DefaultGame.cpp
 *
 *  Created on: 22 Jun 2013
 *      Author: Ashley
 */

#include "DefaultGame.h"


DefaultGame::DefaultGame(string title): StateBasedGame(title) {

}

void DefaultGame::initStates(GameContainer* container) {
	
	// load! yay
	backdrop = new Backdrop();

	font = Resource::get("font.fnt")->asFont()->asBMFont();
	ARK2D::getRenderer()->setFont(font);

	stateLoading = new MyLoadingState(this);
	addState(stateLoading); 
	setCurrentState(stateLoading);
	ARK2D::getContainer()->getTimer()->flush();
	enterState(stateLoading);  

	
}

void DefaultGame::load() {


	stateBlank = new BlankState();
	stateSplash = new SplashState(); 
	stateMenu = new MenuState(); 
	stateInstructions = new InstructionsState();
	stateInGame = new InGameState();  
	stateSummary = new SummaryState(); 



	
	

	addState(stateBlank);
	addState(stateSplash); 
	addState(stateMenu);
	addState(stateInstructions);
	addState(stateInGame);
	addState(stateSummary);
	setCurrentState(stateLoading);

	stateLoading->m_nextState = stateMenu; 

	vector<GameState*> states = getStates();
	for(unsigned int i = 0; i < states.size(); i++) {
		GameState* s = states.at(i);
		if (s != stateLoading) {
			ARK2D::getLog()->i("Initting a state");
			s->init(ARK2D::getContainer(), this);
		}
	} 

	
}

void DefaultGame::update(GameContainer* container, GameTimer* timer) {
	StateBasedGame::update(container, timer);

	

	   


	Input* i = ARK2D::getInput();
	if (i->isGamepadButtonPressed(Gamepad::BUTTON_ACTIVATE) || i->isKeyPressed(Input::KEY_F)) {
		container->setFullscreen(!container->isFullscreen());
	}  

	if (stateLoading->m_doneLoading) {
		backdrop->update(container, timer);
		if (i->isKeyPressed(Input::KEY_M)) {
			float newVol = (SoundStore::getInstance()->getVolumeByGroupId(0)==1.0f)?0.0f:1.0f;
			SoundStore::getInstance()->setVolumeByGroupId(0, newVol);
		}
		 
		if (m_music != NULL && !m_music->isPlaying()) {
			m_music->play();
		}
		
	} 
}

void DefaultGame::drawButton(int x, int y, int width) {
	m_buttonLeft->setColor(backdrop->getTint());
	m_buttonRight->setColor(backdrop->getTint());
	m_buttonTile->setColor(backdrop->getTint());

	m_buttonLeft->drawCentered(x - (width/2), y);
	m_buttonRight->drawCentered(x + (width/2), y);
	m_buttonTile->drawCenteredScaled(x, y, (width-6)/2.0f, 1.0f);
}
 
void DefaultGame::render(GameContainer* container, Renderer* r) {
	font->getImage()->setColor(backdrop->getTint());
	StateBasedGame::render(container, r);

	
}

void DefaultGame::resize(GameContainer* container, int width, int height) {
	StateBasedGame::resize(container, width, height);
}

DefaultGame* DefaultGame::getInstance() {
    return dynamic_cast<DefaultGame*>(ARK2D::getGame());
}

DefaultGame::~DefaultGame() {

}
