/*
 * Backdrop.cpp
 *
 *  Created on: 18 April 2012
 *      Author: Ashley
 */

#include "Backdrop.h"
#include "../DefaultGame.h"

#include "Player.h"

void BackdropStar_set(BackdropStar* star) {
	GameContainer* container = ARK2D::getContainer();
	float f = MathUtil::randBetweenf(0.0f, 1.0f);
	float speed = 0.0f;
	if (f < 0.30f) {
		star->type = 0;
	} else if (f < 0.60f) {
		star->type = 1;
	} else {
		star->type = 2;
	}
	star->rot = MathUtil::randBetweenf(0.0f, 360.0f);
	star->diff = BackdropStar_getDiff(star->type);
	star->speed = BackdropStar_getSpeed(star->type);
	star->speed = MathUtil::randBetweenf(star->speed - 10, star->speed + 10); 
	star->x = MathUtil::randBetweenf(160 - star->diff, 160 + star->diff);
	star->y = MathUtil::randBetweenf(0, container->getHeight());
}
Image* BackdropStar_getImage(BackdropStar* star) {
	if (star->type == 0) {
		return DefaultGame::getInstance()->backdrop->m_star1;
	} else if (star->type == 1) {
		return DefaultGame::getInstance()->backdrop->m_star2;
	} else if (star->type == 2) {
		return DefaultGame::getInstance()->backdrop->m_star3;
	} 
	return DefaultGame::getInstance()->backdrop->m_star1;
}
float BackdropStar_getImageAlpha(BackdropStar* star) {
	if (star->type == 0) {
		return 0.25f;
	} else if (star->type == 1) {
		return 0.5f;
	} else if (star->type == 2) {
		return 0.75f;
	} 
	return 1.0f;
}
float BackdropStar_getDiff(unsigned int t) {
	if (t == 0) {
		return 20;
	} else if (t == 1) {
		return 100;
	} else if (t == 2) {
		return 140;
	} 
	return 0;
}
float BackdropStar_getSpeed(unsigned int t) {
	if (t == 0) {
		return 20.0f;
	} else if (t == 1) {
		return 120.0f;
	} else if (t == 2) {
		return 240.0f;
	} 
	return 0;
}

Backdrop::Backdrop():
	m_index(0),
	m_gradients(),
	m_tints(),
	m_tints50a() {
	
}
void Backdrop::init() {
	DefaultGame* dg = DefaultGame::getInstance();
	
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-1.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-2.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-3.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-8.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-4.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-5.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-6.png")) );
	m_gradients.push_back( dg->spritesheet->getSubImage(dg->desc->getItemByName("background-7.png")) );

	m_tints.push_back( new Color(Color::white) ); 
	m_tints.push_back( new Color(Color::white) ); 
	m_tints.push_back( new Color(Color::white) ); 
	m_tints.push_back( new Color(Color::white) );
	m_tints.push_back( new Color(Color::black) ); 
	m_tints.push_back( new Color(Color::black) ); 
	m_tints.push_back( new Color(Color::black) ); 
	m_tints.push_back( new Color(Color::black) ); 

	m_tints50a.push_back( new Color(Color::white_50a) );
	m_tints50a.push_back( new Color(Color::white_50a) );
	m_tints50a.push_back( new Color(Color::white_50a) );
	m_tints50a.push_back( new Color(Color::white_50a) );
	m_tints50a.push_back( new Color(Color::black_50a) );
	m_tints50a.push_back( new Color(Color::black_50a) );
	m_tints50a.push_back( new Color(Color::black_50a) );
	m_tints50a.push_back( new Color(Color::black_50a) );
	
	m_star1 = dg->spritesheet->getSubImage(dg->desc->getItemByName("star-1.png"));
	m_star2 = dg->spritesheet->getSubImage(dg->desc->getItemByName("star-2.png"));
	m_star3 = dg->spritesheet->getSubImage(dg->desc->getItemByName("star-3.png"));

	m_star1->scale(2.0f, 2.0f);
	m_star2->scale(2.0f, 2.0f);
	m_star3->scale(2.0f, 2.0f);
	m_star1->setCenterOfRotation(m_star1->getWidth()/2.0f, m_star1->getHeight()/2.0f);
	m_star2->setCenterOfRotation(m_star2->getWidth()/2.0f, m_star2->getHeight()/2.0f);
	m_star3->setCenterOfRotation(m_star3->getWidth()/2.0f, m_star3->getHeight()/2.0f);

	m_stars = new vector<BackdropStar*>();
	for(unsigned int i = 0; i < 20; ++i) {
		BackdropStar* st = new BackdropStar();
		BackdropStar_set(st);
		m_stars->push_back(st);

	}
}

Color* Backdrop::getTint() {
	return m_tints.at(m_index);
}
Color* Backdrop::getTint50a() {
	return m_tints50a.at(m_index);
}

void Backdrop::next() {
	m_index++;
	if (m_index == 8) {
		m_index = 7;
	}
}

void Backdrop::update(GameContainer* container, GameTimer* timer) {

	Input* in = ARK2D::getInput();
	if (in->isKeyPressed(Input::KEY_I)) {
		m_index = MathUtil::randBetween(0, 8);
	}

	DefaultGame* dg = DefaultGame::getInstance();
	InGameState* igs = dg->stateInGame;
	Player* pl = DefaultGame::getInstance()->stateInGame->m_player;
	float speedMod = 1.0f * igs->m_gameSpeed;
	if (pl != NULL) {
		speedMod = pl->getBackdropSpeedModifier() * igs->m_gameSpeed;
	}

	for(unsigned int i = 0; i < m_stars->size(); ++i) {
		BackdropStar* st = m_stars->at(i);
		st->y -= st->speed * speedMod * timer->getDelta() ;
		st->rot += st->speed * speedMod * timer->getDelta();
		if (st->y - 5 < 0) {
			st->x = MathUtil::randBetweenf(160 - st->diff, 160 + st->diff);
			st->y = container->getHeight() + 5;
			st->speed = BackdropStar_getSpeed(st->type);
			st->speed = MathUtil::randBetweenf(st->speed - 10, st->speed + 10); 
		}
	}

} 

void Backdrop::render(GameContainer* container, Renderer* r) {
	m_gradients.at(m_index)->draw(0, 0, container->getWidth(), container->getHeight()); 

	for(unsigned int i = 0; i < m_stars->size(); ++i) {
		BackdropStar* st = m_stars->at(i);
		Image* img = BackdropStar_getImage(st);
		img->setColor(getTint());
		img->setAlpha(BackdropStar_getImageAlpha(st));
		img->setRotation(st->rot);
		img->drawCentered(st->x, st->y);
	}
}


Backdrop::~Backdrop() {
	
}
