/*
 * GlassShard.cpp
 *
 *  Created on: Oct 1, 2011
 *      Author: ashleygwinnell
 */

#include "GlassShard.h"
#include "../DefaultGame.h"
#include "Player.h"

GlassShard::GlassShard():  
	Object(),
	m_polygon(NULL),
	m_rotationalVelocity(0.0f)
{
	m_polygon = new ARK::Geometry::Polygon<float>();
	m_polygon->addPoint(0.0f, 0.0f);
	m_polygon->addPoint(0.0f, 0.0f);
	m_polygon->addPoint(0.0f, 0.0f);

	m_bounds = new ARK::Geometry::Rectangle<float>(0, 0, 1, 1);
}

void GlassShard::reset() {

}

void GlassShard::update(GameContainer* container, GameTimer* timer) {
	InGameState* igs = DefaultGame::getInstance()->stateInGame;
	m_polygon->add(m_velocity->getX() * timer->getDelta() * igs->m_gameSpeed, m_velocity->getY() * timer->getDelta() * igs->m_gameSpeed);

	m_polygon->rotate(m_rotationalVelocity * timer->getDelta() * igs->m_gameSpeed);

	if (m_polygon->getMaxX() < DefaultGame::getInstance()->stateInGame->m_cameraX * -1) {
		setPendingRemoval(true);
	}

	
	if (m_polygon->getMaxY() < igs->m_player->m_bounds->getCenterY() - 480) {
		setPendingRemoval(true);
	}
}

void GlassShard::render(GameContainer* container, Renderer* g) {

	DefaultGame* dg = DefaultGame::getInstance();
	Color* cc = DefaultGame::getInstance()->backdrop->getTint50a();

	float summaryFade = 1.0f;
	if (dg->getCurrentState() == dg->stateSummary) {
		if (dg->stateSummary->m_outroTimer > 0.0f) { 
			summaryFade = Easing::ease(Easing::QUADRATIC_OUT, dg->stateSummary->m_outroTimer, 1.0f, -1.0f, dg->stateSummary->m_outroDuration);
		}
	}

	g->setDrawColorf(cc->getRedf(), cc->getGreenf(), cc->getBluef(), cc->getAlphaf() * summaryFade);
	g->fillTriangle( 
		m_polygon->getPoint(0)->getX(), m_polygon->getPoint(0)->getY(),
		m_polygon->getPoint(1)->getX(), m_polygon->getPoint(1)->getY(),
		m_polygon->getPoint(2)->getX(), m_polygon->getPoint(2)->getY()
	); 
}

string GlassShard::toString() {
	return "{}"; 
}
void GlassShard::collision(Object* other) { 

} 


GlassShard::~GlassShard() {
	delete m_polygon;
}