/*
 * LineSection.cpp
 *
 *  Created on: Aug 29, 2012
 *      Author: ashleygwinnell
 */

#include "LineSection.h"
#include "../DefaultGame.h"
#include "Player.h"
#include "GlassShard.h"
 

Image* LineSection::s_endPiece = NULL;

LineSection::LineSection(): 
	Object(), 
	m_line(NULL)
{ 
	m_bounds = new ARK::Geometry::Rectangle<float>(0, 0, 1, 1);
	m_line = new Line<float>(0,0,0,0);
	m_smashed = false;

	if (s_endPiece == NULL) {
		DefaultGame* dg = DefaultGame::getInstance();
		
		s_endPiece = dg->spritesheet->getSubImage(dg->desc->getItemByName("line-side.png"))->scale(2.0, 2.0f);
		s_endPiece->setCenterOfRotation(s_endPiece->getWidth()/2, s_endPiece->getHeight()/2);
	}

	reset();
}

void LineSection::reset() {
	m_smashed = false;

	m_smashSideTimer = 0.0f;
	m_smashSideDuration = 0.5f;

	m_moveSpeed = 0.0f;
}


void LineSection::smashSides() {
	m_smashSideTimer = 0.001f;
}

void LineSection::smash() {
	if (m_smashed) { return; }

	m_smashed = true;
	
	//int layers = 1;
	int xcolumns = m_line->getWidth() / 2;
	int ycolumns = 0;
	//void MenuState::split(int layers, int xcolumns, int ycolumns) { // , int subdivisions) {
	
	//m_testRectangleSplit = true;
	//m_testRectanglePieces->clear();

	// TODO layers.

	//int eachypiece = m_line->getHeight() / (ycolumns+1);

	vector<GlassShard*> thisShards;


	vector<int> randomys;
	randomys.push_back( m_line->getStart()->getY() - 2); //randomys.push_back(m_line->getMinY()-2);
	for(unsigned int i = 0; i < (unsigned int) ycolumns; i++) {
		randomys.push_back( MathUtil::randBetween(int(m_line->getMinY()), int(m_line->getMaxY()))  );
	}
	randomys.push_back( m_line->getStart()->getY() + 2); // randomys.push_back(m_line->getMaxY()+2);
	std::sort(randomys.rbegin(), randomys.rend());

	int lineMinX = m_line->getStart()->getX() - m_line->getWidth(); // m_line->getStart()->getX(); // m_line->getMinX();
	int lineMaxX = m_line->getEnd()->getX(); // m_line->getMaxX();

	for(unsigned int j = 0; j <= (unsigned int) ycolumns; j++) {

		//int thisminY = m_line->getMinY() + (eachypiece * j);
		//int thismaxY = m_line->getMinY() + (eachypiece * (j+1));

		int thisminY = randomys.at(j);
		int thismaxY = randomys.at(j+1);

		vector<int> topxs;
		vector<int> bottomxs;

		topxs.push_back(lineMinX); // topxs.push_back(m_line->getMinX());
		bottomxs.push_back(lineMinX); // bottomxs.push_back(m_line->getMinX());
		for(unsigned int i = 0; i < (unsigned int) xcolumns; i++) {
			topxs.push_back( MathUtil::randBetween(lineMinX, lineMaxX) );
			bottomxs.push_back( MathUtil::randBetween(lineMinX, lineMaxX) );
		}
		topxs.push_back(lineMaxX);
		bottomxs.push_back(lineMaxX);

		std::sort(topxs.rbegin(), topxs.rend());
		std::sort(bottomxs.rbegin(), bottomxs.rend());

		int roottopx = 0;
		int rootbottomx = 0;

		bool first = true;
		int count = 0;
		//while (roottopx <= (signed int) topxs.size() || ) 
		//while (m_testRectanglePieces->size() < (unsigned int) ((xcolumns*2) + 2) * (j+1) ) 
		while (count < xcolumns+2)
		{
			GlassShard* gs = DefaultGame::getInstance()->stateInGame->m_glassShards->get(); //new GlassShard();
			gs->reset(); 
			if (first) {
				gs->m_polygon->getPoint(0)->set(topxs.at(roottopx), thisminY);
				gs->m_polygon->getPoint(1)->set(bottomxs.at(rootbottomx), thismaxY);
				gs->m_polygon->getPoint(2)->set(bottomxs.at(rootbottomx+1), thismaxY);
				
				rootbottomx++;
				first = false; 
			} else {
				gs->m_polygon->getPoint(0)->set(bottomxs.at(rootbottomx), thismaxY);
				gs->m_polygon->getPoint(1)->set(topxs.at(roottopx), thisminY);
				gs->m_polygon->getPoint(2)->set(topxs.at(roottopx+1), thisminY);
				
				roottopx++;
				first = true;
			}
			count++;
			thisShards.push_back(gs);
			//m_testRectanglePieces->add(gs);
		}

	}


	
//}
	
	// give each piece a velocity.
	//Input* input = ARK2D::getInput();
	Player* player = DefaultGame::getInstance()->stateInGame->m_player;
	int mx = player->m_bounds->getCenterX(); //input->getMouseX();
	int my = player->m_bounds->getCenterY(); //input->getMouseY();
	for(unsigned int i = 0 ; i < thisShards.size(); i++) {
		GlassShard* gs = thisShards.at(i);
		gs->m_rotationalVelocity = MathUtil::randBetween(0, 180);
		if (MathUtil::randBetween(0, 2) == 0) {
			gs->m_rotationalVelocity  *= -1;
		}

		double anglef = MathUtil::anglef(mx, my, gs->m_polygon->getCenterX(), gs->m_polygon->getCenterY());
		MathUtil::moveAngle<float>(gs->m_velocity, anglef, (float) MathUtil::randBetween(10, 100));
	}

	thisShards.clear();
}



void LineSection::update(GameContainer* container, GameTimer* timer) 
{
	InGameState* igs = DefaultGame::getInstance()->stateInGame;
	//m_line->add(igs->m_speed * timer->getDelta() * -1, 0);

	if(m_smashSideTimer > 0.0f) {
		m_smashSideTimer += timer->getDelta() * igs->m_gameSpeed;
		if (m_smashSideTimer > m_smashSideDuration) {
			m_smashSideTimer = m_smashSideDuration;
		}
	}

	if (m_moveSpeed != 0.0f) {
		m_line->setStartX(m_line->getMinX() + m_moveSpeed * timer->getDelta() * igs->m_gameSpeed);
		m_line->setEndX(m_line->getMaxX() + m_moveSpeed * timer->getDelta() * igs->m_gameSpeed);

		if (m_moveSpeed > 0.0f && m_line->getMaxX() > container->getWidth()) {
			float diff = float(container->getWidth()) - m_line->getMaxX();
			m_line->setStartX(m_line->getMinX() + diff);
			m_line->setEndX(m_line->getMaxX() + diff);
			m_moveSpeed *= -1;
		} else if (m_moveSpeed < 0.0f && m_line->getMinX() < 0) {
			float diff = m_line->getMinX() * -1.0f;
			m_line->setStartX(m_line->getMinX() + diff);
			m_line->setEndX(m_line->getMaxX() + diff);
			m_moveSpeed *= -1;
		}
	}

	if (m_line->getMinY() < igs->m_player->m_bounds->getCenterY() - 480) {
		setPendingRemoval(true);
	}
}
void LineSection::render(GameContainer* container, Renderer* g) {
	
	
	if (!m_smashed) { 
		g->setLineWidth(2);
	
		//g->setDrawColor(Color::white_50a);
		g->setDrawColor(DefaultGame::getInstance()->backdrop->getTint50a());
	
	//	if (DefaultGame::getInstance()->stateInGame->m_player->m_nearestLine == this) {
		//	g->setDrawColor(Color::red);		
	//	}

		g->drawLine(m_line->getStart()->getX() , m_line->getStart()->getY() , m_line->getEnd()->getX() , m_line->getEnd()->getY() );
		g->setLineWidth(1);
	}

	float sideScale = 1.0f;
	if (m_smashSideTimer > 0.0f) {
		sideScale = Easing::ease(Easing::QUADRATIC_OUT, m_smashSideTimer, 1.0f, -1.0f, m_smashSideDuration);
	} 

	s_endPiece->setColor(DefaultGame::getInstance()->backdrop->getTint());
	s_endPiece->drawCenteredScaled(m_line->getStart()->getX(), m_line->getStart()->getY(), sideScale, sideScale);
	s_endPiece->drawCenteredScaled(m_line->getEnd()->getX(), m_line->getEnd()->getY(), sideScale, sideScale);

	if (ARK2D::getLog()->isVisible()) {
		g->setDrawColor(Color::yellow);
		g->fillCircle(m_line->getMinX(), m_line->getMinY(), 10, 20);
		g->fillCircle(m_line->getMaxX(), m_line->getMinY(), 10, 20);
	}
}

string LineSection::toString() {
	string s("");
	s += "{";
	s += " \"line\":";
	s += m_line->toString();
	s += ",";
	s += " \"type\":";
	//s += Cast::toString<unsigned int>(m_type);
	s += "}";
	return s;
}

void LineSection::collision(Object* other) { 

} 


LineSection::~LineSection() {

}

