/*
 * Player.cpp
 *
 *  Created on: 29 Jan 2011
 *      Author: Ashley
 */

#include "Player.h"
#include "../DefaultGame.h"

#include "Particle.h"
#include "LineSection.h"
 
Image* Player::s_spr = NULL;

Player::Player(): 
	Object()
	{

	DefaultGame* dg = DefaultGame::getInstance(); 

	m_bounds = new ARK::Geometry::Rectangle<float>(0, 0, 26, 26);
	
  	//m_velocityMax->set(250, 2000);  //accelerative  
  	m_velocityMax->set(250, 500);    
	m_acceleration->set(400, 300); 

	if (s_spr == NULL) {
		s_spr = dg->spritesheet->getSubImage(dg->desc->getItemByName("player-2.png"))->scale(2.0, 2.0f);
		s_spr->setCenterOfRotation(s_spr->getWidth()/2, s_spr->getHeight()/2);
	}

	m_spr = s_spr->getCopy();
	m_spr->setCenterOfRotation(m_spr->getWidth()/2, m_spr->getHeight()/2);
	
	reset();
}   
   
void Player::reset() 
{
	m_bounds->setLocationByCenter(160, -50);
	m_velocity->set(0, 0.0f);  
	//m_velocity->set(0, m_velocityMax->getY()/2.0f);  

	m_rotation = 0.0f;
	m_rotationSpeed = 360.0f;

	m_alive = true;
	m_yvelatdeath = 0;
	
	m_deathTimer = 0.0f;
	m_deathDuration = 1.0f;

	m_killedBy = NULL;
}  

void Player::die() {
	DefaultGame* dg = DefaultGame::getInstance();
	InGameState* igs = dg->stateInGame;

	dg->m_sfxDie->play();

	m_alive = false;
	m_yvelatdeath = m_velocity->getY();
	igs->m_outroTimer = 0.001f;

	// smash all the line sections up, yo!!
	PoolIterator<LineSection* >* it_lines = igs->m_lines->iterator();
	while (it_lines->hasNext()) {
		LineSection* ls = it_lines->next();
		//if (ls != m_killedBy) { 
			ls->smash();
			ls->smashSides();
		//}

	}

	if (igs->m_score > igs->m_scoreHigh) {
		igs->m_scoreHigh = igs->m_score;
	}

	float dist = 7;

	
	MyParticle* p = igs->m_particles->get();
	p->reset();
	p->start(MyParticle::TYPE_PLAYER_GIB);
	p->m_bounds->setLocationByCenter(m_bounds->getCenterX() - dist, m_bounds->getCenterY() - dist);

	p = igs->m_particles->get();
	p->reset();
	p->start(MyParticle::TYPE_PLAYER_GIB);
	p->m_bounds->setLocationByCenter(m_bounds->getCenterX() - dist, m_bounds->getCenterY() + dist);

	p = igs->m_particles->get();
	p->reset();
	p->start(MyParticle::TYPE_PLAYER_GIB);
	p->m_bounds->setLocationByCenter(m_bounds->getCenterX() + dist, m_bounds->getCenterY() - dist);

	p = igs->m_particles->get();
	p->reset();
	p->start(MyParticle::TYPE_PLAYER_GIB);
	p->m_bounds->setLocationByCenter(m_bounds->getCenterX() + dist, m_bounds->getCenterY() + dist);

	p = igs->m_particles->get();
	p->reset();
	p->start(MyParticle::TYPE_PLAYER_GIB);
	p->m_bounds->setLocationByCenter(m_bounds->getCenterX(), m_bounds->getCenterY());

	dg->enterState(dg->stateSummary);
}

void Player::update(GameContainer* container, GameTimer* timer) {
	InGameState* igs = DefaultGame::getInstance()->stateInGame;

	if (m_alive) { 

		gravity();
		
		Input* in = ARK2D::getInput();
		if (in->isKeyDown(Input::KEY_LEFT) && in->isKeyDown(Input::KEY_RIGHT)) {

		} else if (in->isKeyDown(Input::KEY_LEFT)) {
			m_velocity->setX(-300);
			m_facingLeft = true;
		} else if (in->isKeyDown(Input::KEY_RIGHT)) {
			m_velocity->setX(300);
			m_facingLeft = false;
		} else {
			m_velocity->setX(0); 
		}

	} else {
		m_deathTimer += timer->getDelta() * igs->m_gameSpeed;

		float newy = Easing::ease(Easing::QUADRATIC_OUT, m_deathTimer, m_yvelatdeath, m_yvelatdeath*-1, m_deathDuration);
		m_velocity->set(0, newy);
		
	}
	
	m_rotation += m_rotationSpeed * timer->getDelta() * igs->m_gameSpeed;

	m_bounds->asRectangle()->setLocationByCenter( 
		m_bounds->getCenterX() + (m_velocity->getX() * timer->getDelta() * igs->m_gameSpeed),
		m_bounds->getCenterY() + (m_velocity->getY() * timer->getDelta() * igs->m_gameSpeed)
	);

	keepInBounds();
}
void Player::collision(Object* other) { 

} 

float Player::getBackdropSpeedModifier() {
	float val = 0.5f;
	val += 1.0f * (m_velocity->getY() / m_velocityMax->getY());
	return val;
}


void Player::render(GameContainer* container, Renderer* r) 
{
	if (m_alive) { 
		m_spr->setColor(DefaultGame::getInstance()->backdrop->getTint());
		m_spr->setRotation(m_rotation);
		m_spr->drawCentered(m_bounds->getCenterX(), m_bounds->getCenterY());	
		//m_spr->drawCenteredFlipped(m_bounds->getCenterX(), m_bounds->getCenterY(), m_facingLeft, false);	
	}

	if (ARK2D::getLog()->isVisible()) {
		r->setDrawColor(Color::red);
		r->fillCircle(m_bounds->getCenterX(), m_bounds->getCenterY(), 8, 20);
	}


}

Player::~Player() { 
	 
}
