/*
 * InGameState.cpp
 *
 *  Created on: 08 Jun 2011
 *      Author: Ashley
 */

#include "InGameState.h"
#include "../DefaultGame.h"

#include "../objects/Object.h"
#include "../objects/BlankObject.h"
#include "../objects/Player.h"
#include "../objects/Pickup.h"
#include "../objects/Explosion.h"

#include "../objects/LineSection.h"
#include "../objects/GlassShard.h"

#include "../objects/Particle.h"
 
InGameState::InGameState(): 
	GameState(),  
	m_player(NULL),
	m_pickups(NULL),
	m_explosions(NULL),

	content_lineStartMin(),
	content_lineStartMax(),
	content_lineWidthMin(),
	content_lineWidthMax(),
	content_lineMoveSpeed(),
	content_playerMaxVelocityY(),
	content_fixedPattern()

	{

}

unsigned int InGameState::id() {
	return StatesList::STATE_INGAME;
}

void InGameState::enter(GameContainer* container, StateBasedGame* game, GameState* from) { 

}
void InGameState::leave(GameContainer* container, StateBasedGame* game, GameState* to) { 
	m_gameSpeed = 1.0f; 
}

void InGameState::init(GameContainer* container, StateBasedGame* game) 
{
	m_player = new Player();
	m_pickups = new Pool<Pickup*>(5);
	m_explosions = new Pool<Explosion*>(5);

	m_lines = new Pool<LineSection*>(20);
	m_glassShards = new Pool<GlassShard*>(400);
	m_particles = new Pool<MyParticle*>(20);

	m_cameraRect = new ARK::Geometry::Rectangle<float>(0, 0, container->getWidth(), container->getHeight());
	m_cameraX = 0; 
	m_cameraY = 0; 
	m_cameraTargetX = m_cameraX; 
	m_cameraTargetY = m_cameraY;

	m_cameraZoom = 1.0f;
	m_cameraRotate = 0.0f;
	m_cameraZoomTimer = new Timer(0.5f);
	m_cameraRotateTimer = new Timer(0.5f);

	m_introTimer = 0.001f;
	m_introDuration = 1.0f;

	// wave 1?
	content_lineStartMin.push_back(0);
	content_lineStartMax.push_back(160);
	content_lineWidthMin.push_back(120);
	content_lineWidthMax.push_back(160);
	content_lineMoveSpeed.push_back(0);
	content_playerMaxVelocityY.push_back(500);
	content_fixedPattern.push_back(false);

	// wave 2...
	content_lineStartMin.push_back(0);
	content_lineStartMax.push_back(180);
	content_lineWidthMin.push_back(100);
	content_lineWidthMax.push_back(140);
	content_lineMoveSpeed.push_back(0);
	content_playerMaxVelocityY.push_back(550);
	content_fixedPattern.push_back(false);

	// wave 3......
	content_lineStartMin.push_back(0);
	content_lineStartMax.push_back(220);
	content_lineWidthMin.push_back(80);
	content_lineWidthMax.push_back(100);
	content_lineMoveSpeed.push_back(0);
	content_playerMaxVelocityY.push_back(600);
	content_fixedPattern.push_back(false);

	// wave 4?! // moving platforms
	content_lineStartMin.push_back(0);
	content_lineStartMax.push_back(160);
	content_lineWidthMin.push_back(100);
	content_lineWidthMax.push_back(140);
	content_lineMoveSpeed.push_back(100);
	content_playerMaxVelocityY.push_back(550);
	content_fixedPattern.push_back(false);

	// wave 5?! smaller lines and moving
	content_lineStartMin.push_back(0);
	content_lineStartMax.push_back(220);
	content_lineWidthMin.push_back(80);
	content_lineWidthMax.push_back(100);
	content_lineMoveSpeed.push_back(80);
	content_playerMaxVelocityY.push_back(550);
	content_fixedPattern.push_back(false);

	// wave 6, what comes after?
	m_scoreHigh = 0;

	reset();
}


void InGameState::start() { 
	//reset();
	m_explosions->reset();
	m_player->m_velocityMax->setY(content_getPlayerMaxVelocityY());

	m_score = 0;
	DefaultGame* dg = DefaultGame::getInstance();
	dg->backdrop->m_index = 0;
}
 
void InGameState::reset() {
	ARK2D::getLog()->i("InGameState::reset");

	m_player->reset();
	m_pickups->reset();
	m_glassShards->reset();
	m_lines->reset();
	m_particles->reset();

	m_score = 0;

	m_cameraX = 0; 
	m_cameraY = 0; 
	m_cameraTargetX = m_cameraX; 
	m_cameraTargetY = m_cameraY;

	m_generateLineTimer = 0.0f;
	m_generateLineDuration = 0.5f;

	m_gameSpeed = 1.0f;
	m_gameSpeedSlowdownTimer = 0.0f;
	m_gameSpeedSlowdownDuration = 1.0f;

	m_introTimer = 0.001f;
	m_introDuration = 1.0f;

	m_outroTimer = 0.0f;
	m_outroDuration = 1.0f;

	m_flashTimer = 0.0f;
	m_flashDuration = 1.0f;

	DefaultGame* dg = DefaultGame::getInstance();
	//if (dg->backdrop->m_index != 0) {
	//	dg->backdrop->m_index = 0;
	//	
	//}
	flash();
}

void InGameState::flash() {
	m_flashTimer = 0.001f;
}

void InGameState::updateCameraVars(float sc) 
{
	if (m_cameraZoomTimer->update(m_gameSpeed)) {
		m_cameraZoomTimer->loop();
	}
	if (m_cameraRotateTimer->update(m_gameSpeed)) { 
		m_cameraRotateTimer->loop();
	}

	// ZOOM
	float cameraZoomAmount = 0.00f;
	//if (m_cameraNauseating) { cameraZoomAmount = 0.2f; }
	cameraZoomAmount *= sc;

	float cameraZoomHalf = m_cameraZoomTimer->getDuration()/2.0f; 
	if (m_cameraZoomTimer->getTime() < cameraZoomHalf) {
		m_cameraZoom = Easing::ease(Easing::QUADRATIC_IN_OUT, m_cameraZoomTimer->getTime(), 1.0f, cameraZoomAmount, cameraZoomHalf);
	} else { 
		m_cameraZoom = Easing::ease(Easing::QUADRATIC_IN_OUT, m_cameraZoomTimer->getTime()-cameraZoomHalf, 1.0f + cameraZoomAmount, cameraZoomAmount*-1.0f, cameraZoomHalf);
	}  

	// ROTATE
	float cameraRotateAmount = 0.00f;
//	if (m_cameraNauseating) { cameraRotateAmount = 2.0f; }
	cameraRotateAmount *= sc;

	float cameraRotateQuarter = m_cameraRotateTimer->getDuration()/4.0f; 
	if (m_cameraRotateTimer->getTime() < cameraRotateQuarter) {
		m_cameraRotate = Easing::ease(Easing::QUADRATIC_OUT, m_cameraRotateTimer->getTime(), 0.0f, cameraRotateAmount, cameraRotateQuarter);
	} else if (m_cameraRotateTimer->getTime() < 2*cameraRotateQuarter) { 
		m_cameraRotate = Easing::ease(Easing::QUADRATIC_IN, m_cameraRotateTimer->getTime()-cameraRotateQuarter, cameraRotateAmount, cameraRotateAmount * -1.0f, cameraRotateQuarter);
	} else if (m_cameraRotateTimer->getTime() < 3*cameraRotateQuarter) { 
		m_cameraRotate = Easing::ease(Easing::QUADRATIC_OUT, m_cameraRotateTimer->getTime()-(2*cameraRotateQuarter), 0.0f, cameraRotateAmount * -1.0f, cameraRotateQuarter);
	} else { 
		m_cameraRotate = Easing::ease(Easing::QUADRATIC_IN, m_cameraRotateTimer->getTime()-(3*cameraRotateQuarter), cameraRotateAmount * -1.0f, cameraRotateAmount, cameraRotateQuarter);
	}  
}

float InGameState::content_getLineStartMin() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_lineStartMin.size()) { index = content_lineStartMin.size() - 1; }
	return content_lineStartMin.at(index);
}
float InGameState::content_getLineStartMax() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_lineStartMax.size()) { index = content_lineStartMax.size() - 1; }
	return content_lineStartMax.at(index);
}
float InGameState::content_getLineWidthMin() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_lineWidthMin.size()) { index = content_lineWidthMin.size() - 1; }
	return content_lineWidthMin.at(index);
}
float InGameState::content_getLineWidthMax() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_lineWidthMax.size()) { index = content_lineWidthMax.size() - 1; }
	return content_lineWidthMax.at(index);
}

float InGameState::content_getLineMoveSpeed() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_lineMoveSpeed.size()) { index = content_lineMoveSpeed.size() - 1; }
	return content_lineMoveSpeed.at(index);
}
float InGameState::content_getPlayerMaxVelocityY() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_playerMaxVelocityY.size()) { index = content_playerMaxVelocityY.size() - 1; }
	return content_playerMaxVelocityY.at(index);
}
bool InGameState::content_isFixedPattern() {
	DefaultGame* dg = DefaultGame::getInstance();
	unsigned int index = dg->backdrop->m_index;
	if (index >= content_fixedPattern.size()) { index = content_fixedPattern.size() - 1; }
	return content_fixedPattern.at(index);	
}

void InGameState::update(GameContainer* container, StateBasedGame* game, GameTimer* timer) {
	DefaultGame* dg = DefaultGame::getInstance();
	Input* in = ARK2D::getInput();

	if (in->isKeyPressed(Input::KEY_ESCAPE) || in->isKeyPressed(Input::KEY_ENTER) || in->isGamepadButtonPressed(Gamepad::BUTTON_START)) {
	//if (i->isKeyPressed(Input::KEY_ENTER) ||  || i->isGamepadButtonPressed(Gamepad::BUTTON_A)) {
		//dg->sfx_ui_select->play();
		if (dg->getCurrentState() == dg->stateInGame) {
			game->enterState(dg->stateMenu);
		}
	}

	// intro timer
	if (m_introTimer > 0.0f) { 
		m_introTimer += timer->getDelta();
		if (m_introTimer > m_introDuration) {
			m_introTimer = 0.0f;
		}
	}

	// outro timer (for vanity sake)
	if (m_outroTimer > 0.0f) { 
		m_outroTimer += timer->getDelta();
		if (m_outroTimer > m_outroDuration) {
			m_outroTimer = m_outroDuration; 
		}
	}

	// updat egame speed
	m_gameSpeed = 1.0f;
	if (m_gameSpeedSlowdownTimer > 0.0f) {
		m_gameSpeedSlowdownTimer += timer->getDelta();
		if (m_gameSpeedSlowdownTimer >= m_gameSpeedSlowdownDuration) {
			m_gameSpeedSlowdownTimer = 0.0f;
		} 
		m_gameSpeed = Easing::ease(Easing::QUADRATIC_IN, m_gameSpeedSlowdownTimer, 0.25f, 0.75f, m_gameSpeedSlowdownDuration);
	}
	m_gameTimer += timer->getDelta() * m_gameSpeed;

	// fllsh timer
	if (m_flashTimer > 0.0f) {
		m_flashTimer += timer->getDelta();
		if (m_flashTimer >= m_flashDuration) {
			m_flashTimer = 0.0f;
		}
	}

	if (m_introTimer == 0.0f) { 
		if (m_player->m_alive) { 
			m_generateLineTimer += timer->getDelta() * m_gameSpeed;
			if (m_generateLineTimer >= m_generateLineDuration) {
				m_generateLineTimer -= m_generateLineDuration;

				float py = m_player->m_bounds->getCenterY() + 360;
				LineSection* ls = m_lines->get();
				ls->reset();
				ls->m_moveSpeed = content_getLineMoveSpeed();

				float startX = MathUtil::randBetweenf(content_getLineStartMin(), content_getLineStartMax());
				float length = MathUtil::randBetweenf(content_getLineWidthMin(), content_getLineWidthMax());

				ls->m_line->setStart(startX, py);
				ls->m_line->setEnd(startX+length, py);
			}
		}
	}

	

 	//updateCameraVars(1.0f);
 	signed int m_levelWidth = 320;
	signed int m_levelHeight = m_player->m_bounds->getCenterY() + 480;

 	// Camera Target
	m_cameraTargetX = 0;
	if (m_player->m_bounds->getCenterX() > m_levelWidth - (container->getWidth()/2)) {
		m_cameraTargetX = (m_levelWidth - (container->getWidth())) * -1;
	} else if (m_player->m_bounds->getCenterX() > container->getWidth()/2) {
		m_cameraTargetX = (container->getWidth()/2) - m_player->m_bounds->getCenterX();
	} 
	m_cameraTargetY = 0; 
	/*if (m_player->m_bounds->getCenterY() > m_levelHeight - (container->getHeight()/2)) {
		m_cameraTargetY = (m_levelHeight - (container->getHeight())) * -1;
	} else if (m_player->m_bounds->getCenterY() > container->getHeight()/2) {
		m_cameraTargetY = (container->getHeight()/2) - m_player->m_bounds->getCenterY();
	} */
	m_cameraTargetY = (m_player->m_bounds->getCenterY() - 120) * -1;


   
	// Chase Camera Target  
	float cameraDistance = MathUtil::distance(m_cameraX, m_cameraY, m_cameraTargetX, m_cameraTargetY);
	float cameraSpeed = Easing::ease(Easing::QUADRATIC_OUT, cameraDistance, 0.0f, m_player->m_velocityMax->getY()*1.5f, 200.0f);
	float newCameraAngle = MathUtil::anglef(m_cameraX, m_cameraY, m_cameraTargetX, m_cameraTargetY);
	float newCameraX = m_cameraX;
	float newCameraY = m_cameraY;
	MathUtil::moveAngle<float>(newCameraX, newCameraY, newCameraAngle, cameraSpeed * timer->getDelta() * m_gameSpeed);

	float newCameraDistance = MathUtil::distance(m_cameraX, m_cameraY, newCameraX, newCameraY);
	if (newCameraDistance > cameraDistance) {
		newCameraX = m_cameraTargetX;
		newCameraY = m_cameraTargetY;
	}

	m_cameraX = newCameraX; 
	m_cameraY = newCameraY;
	m_cameraRect->setLocation(m_cameraX*-1, m_cameraY*-1);

	//m_cameraX += m_cameraShake->getXOffset(); 
	//m_cameraY += m_cameraShake->getYOffset();
 
	
	updateCameraVars(1.0f);

	PoolIterator<Pickup*>* iterate_pickups = m_pickups->iterator();
	while (iterate_pickups->hasNext()) {
		Pickup* p = iterate_pickups->next();
		p->update(container, timer);  
	}

	m_explosions->updateAll(container, timer);
	m_explosions->pruneAll();
 
 	m_lines->updateAll(container, timer);
 	m_lines->pruneAll();

 	m_glassShards->updateAll(container, timer);
	m_glassShards->pruneAll();

	m_particles->updateAll(container, timer);
	m_particles->pruneAll();

	m_player->update(container, timer);

	doCollisions();
	
}
void InGameState::doCollisions() {

	DefaultGame* dg = DefaultGame::getInstance();
	if (m_player->m_alive) 
	{ 

		PoolIterator<LineSection*>* it_lines = m_lines->iterator();
		while (it_lines->hasNext()) {
			LineSection* ls = it_lines->next();

			if (m_player->m_bounds->getMaxY() >= ls->m_line->getMinY() && 
				m_player->m_bounds->getCenterY() < ls->m_line->getMinY() && 
				m_player->m_bounds->getCenterX() > ls->m_line->getMinX() && 
				m_player->m_bounds->getCenterX() < ls->m_line->getMaxX()
				) {

				// smash! ;D
				if (!ls->m_smashed) { 
					m_score++;
					ls->smash();
					dg->m_sfxMenuSwitch->play();

					if (m_score % 10 == 0) { 
						m_gameSpeedSlowdownTimer = 0.001f;
						flash();
						dg->backdrop->next();
					}

					//m_player->m_velocity->setY(m_player->m_velocity->getY() * 0.8f);
				}
				

			}

			bool collidesLeftSide = Shape<float>::collision_circleCircle(m_player->m_bounds->getCenterX(), m_player->m_bounds->getCenterY(), 10, ls->m_line->getMinX(), ls->m_line->getMinY(), 8);
			bool collidesRightSide = Shape<float>::collision_circleCircle(m_player->m_bounds->getCenterX(), m_player->m_bounds->getCenterY(), 10, ls->m_line->getMaxX(), ls->m_line->getMinY(), 8);
			if (collidesLeftSide || collidesRightSide) { 
				m_player->m_killedBy = ls;
				m_player->die();
			}

		}

	}
	
}

void InGameState::render(GameContainer* container, StateBasedGame* game, Renderer* r) {
	//r->drawString("InGame State", 20, 20, Renderer::ALIGN_LEFT, Renderer::ALIGN_TOP);
	DefaultGame* dg = DefaultGame::getInstance();
	dg->backdrop->render(container, r);
	//dg->bg->draw(0, 0, container->getWidth(), container->getHeight()); 
	// Boilerplate
	//r->getBatch()->setEnabled(true);
	//
	//r->getBatch()->render(); 
	//r->getBatch()->setEnabled(false);

	// Explosions
	/*PoolIterator<Explosion*>* iterate_explosions = m_explosions->iterator();
	while (iterate_explosions->hasNext()) {
		Explosion* e = iterate_explosions->next();
		e->render(container, r);  
	} 
	
	// Pickups
	PoolIterator<Pickup*>* iterate_pickups = m_pickups->iterator();
	while (iterate_pickups->hasNext()) {
		Pickup* p = iterate_pickups->next(); 
		p->render(container, r);  
	} */

	//r->pushMatrix();
	//r->translate(0.0f, m_player->m_bounds->getCenterY() - 240.0f);

		startGameCamera(1.0f, 1.0f);

	m_player->render(container, r);

	r->getBatch()->setEnabled(true);
	m_lines->renderAll(container, r);
	m_glassShards->renderAll(container, r);
	r->getBatch()->render();
	r->getBatch()->setEnabled(false);

	m_particles->renderAll(container, r);
	

	endGameCamera();
	//r->popMatrix();

	float scoreY = 30.0f;
	if (m_introTimer > 0.0f) {
		scoreY = Easing::ease(Easing::QUADRATIC_OUT, m_introTimer, -30.0f, 60.0f, m_introDuration);
	} else if (m_outroTimer > 0.0f) {
		scoreY = Easing::ease(Easing::QUADRATIC_OUT, m_outroTimer, 30.0f, -60.0f, m_outroDuration);
	}

	r->drawString(Cast::toString<unsigned int>(m_score), 160, scoreY, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);

	// next world overlay
	if (m_flashTimer > 0.0f) {
		float alpha = Easing::ease(Easing::QUADRATIC_OUT, m_flashTimer, 1.0f, -1.0f, m_flashDuration);
		r->setDrawColorf(1.0f, 1.0f, 1.0f, alpha);
		r->fillRect(0, 0, container->getWidth(), container->getHeight());
	}
}

void InGameState::startGameCamera(float scaleX, float scaleY) {
	Renderer* r = ARK2D::getRenderer();

	r->pushMatrix();  
	r->translate(getCurrentCameraX() * scaleX, getCurrentCameraY() * scaleY);    
 
	r->pushMatrix(); 
	r->translate(m_player->m_bounds->getCenterX(), m_player->m_bounds->getCenterY());
	r->scale(m_cameraZoom, m_cameraZoom);
	r->translate(m_player->m_bounds->getCenterX()*-1, m_player->m_bounds->getCenterY()*-1);

	r->pushMatrix();
	r->translate(m_player->m_bounds->getCenterX(), m_player->m_bounds->getCenterY());
	r->rotate(m_cameraRotate);
	r->translate(m_player->m_bounds->getCenterX()*-1, m_player->m_bounds->getCenterY()*-1);
 
	//r->pushMatrix();
	//r->translate(m_player->m_bounds->getCenterX(), m_player->m_bounds->getCenterY());
	//r->rotate(m_player->getCurrentRotation() - 180);
	//r->translate(m_player->m_bounds->getCenterX()*-1, m_player->m_bounds->getCenterY()*-1);	

	r->pushMatrix();
}
void InGameState::endGameCamera() {
	Renderer* r = ARK2D::getRenderer();
	r->popMatrix(); 
	r->popMatrix();
	
	r->popMatrix();
	r->popMatrix();
}

InGameState::~InGameState() {

}

