/*
 * InGameState.h
 * 08 Jun 2011
 */

#ifndef INGAMESTATE_H_
#define INGAMESTATE_H_

#include "../ARK.h"

class DefaultGame;

class Object;
class BlankObject;
class Player;
class Pickup;
class Explosion;

class LineSection;
class GlassShard;

class MyParticle;


class InGameState : public GameState {
	public:

		Player* m_player;
		Pool<Pickup* >* m_pickups;
		Pool<Explosion* >* m_explosions;
		
		Pool<LineSection* >* m_lines;
		Pool<GlassShard* >* m_glassShards;
		Pool<MyParticle* >* m_particles; 

		ARK::Geometry::Rectangle<float>* m_cameraRect;
		float m_cameraX; 
		float m_cameraY; 
		signed int m_cameraTargetX; 
		signed int m_cameraTargetY; 

		float m_cameraZoom;
		float m_cameraRotate;
		Timer* m_cameraZoomTimer;
		Timer* m_cameraRotateTimer;

		unsigned int m_score;
		unsigned int m_scoreHigh;

		float m_generateLineTimer;
		float m_generateLineDuration;

		float m_gameTimer;

		float m_gameSpeed;
		float m_gameSpeedSlowdownTimer;
		float m_gameSpeedSlowdownDuration;

		float m_introTimer;
		float m_introDuration;

		float m_flashTimer;
		float m_flashDuration;

		float m_outroTimer;
		float m_outroDuration;

		vector<float> content_lineStartMin;
		vector<float> content_lineStartMax;
		vector<float> content_lineWidthMin;
		vector<float> content_lineWidthMax;
		vector<float> content_playerMaxVelocityY;
		vector<float> content_lineMoveSpeed;
		vector<bool> content_fixedPattern;
		
		InGameState();
		void enter(GameContainer* container, StateBasedGame* game, GameState* from);
		void leave(GameContainer* container, StateBasedGame* game, GameState* to);

		float content_getLineStartMin();
		float content_getLineStartMax();
		float content_getLineWidthMin();
		float content_getLineWidthMax();
		float content_getLineMoveSpeed();
		float content_getPlayerMaxVelocityY();
		bool content_isFixedPattern();

		void flash();

		unsigned int id();
		void reset();
		void start();
		void init(GameContainer* container, StateBasedGame* game);
		void update(GameContainer* container, StateBasedGame* game, GameTimer* timer);
		void doCollisions();
		
		float getCurrentCameraX() { return m_cameraX; }// + m_cameraShake->getXOffset(); }
		float getCurrentCameraY() { return m_cameraY; }// + m_cameraShake->getYOffset(); }
		void startGameCamera(float scaleX, float scaleY);
		void endGameCamera();
		void updateCameraVars(float sc);
		
		void render(GameContainer* container, StateBasedGame* game, Renderer* r);

		virtual ~InGameState();
};

#endif /* INGAMESTATE_H_ */
