/*
 * InstructionsState.cpp
 *
 *  Created on: 08 Jun 2011
 *      Author: Ashley
 */

#include "../ARK.h"
#include "InstructionsState.h"

#include "../DefaultGame.h"

InstructionsState::InstructionsState():
	GameState() {

}

unsigned int InstructionsState::id() {
	return StatesList::STATE_INSTRUCTIONS;
}

void InstructionsState::enter(GameContainer* container, StateBasedGame* game, GameState* from) {
	m_introTimer = 0.001f;
	m_outroTimer = 0.0f;
}
void InstructionsState::leave(GameContainer* container, StateBasedGame* game, GameState* to) {

}

void InstructionsState::init(GameContainer* container, StateBasedGame* game) {

	m_introTimer = 0.001f;
	m_introDuration = 0.8f;

	m_outroTimer = 0.0f;
	m_outroDuration = 0.8f;

}

void InstructionsState::update(GameContainer* container, StateBasedGame* game, GameTimer* timer) {
	DefaultGame* dg = DefaultGame::getInstance();
	Input* i = ARK2D::getInput();

	if (i->isKeyPressed(Input::KEY_ENTER)) {
		//game->enterState(dg->stateMenu);
		if (m_introTimer == 0.0f && m_outroTimer == 0.0f) { 
			m_outroTimer = 0.001f;
			dg->m_sfxMenuBack->play();
		}
	}

	//dg->stateInGame->update(container, game, timer);

	if (m_introTimer > 0.0f) {
		m_introTimer += timer->getDelta();
		if (m_introTimer >= m_introDuration) {
			m_introTimer = 0.0f;
		}
	}

	if (m_outroTimer > 0.0f) {
		m_outroTimer += timer->getDelta();
		if (m_outroTimer >= m_outroDuration) {
			dg->enterState(dg->stateMenu);
		}
	}
} 

void InstructionsState::render(GameContainer* container, StateBasedGame* game, Renderer* r) {
	DefaultGame* dg = DefaultGame::getInstance();
	dg->backdrop->render(container, r);

	float logoY = 65;
	if (m_introTimer > 0.0f) {
		logoY = Easing::ease(Easing::QUADRATIC_OUT, m_introTimer, 505, -440.0f, m_introDuration);
	} 
	if (m_outroTimer > 0.0f) {
		logoY = Easing::ease(Easing::QUADRATIC_OUT, m_outroTimer, 65, -480.0f, m_outroDuration);
	}

	int wid = (signed int) container->getWidth();
	r->drawString("INSTRUCTIONS", 160, logoY, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	
	r->drawString("- Fall Forever", wid/2, logoY + 40, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.5f);
	r->drawString("- Break Glass", wid/2, logoY + 80, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.5f);
	r->drawString("- Avoid Edges", wid/2, logoY + 120, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.5f);

	r->drawString("CONTROLS", 160, logoY+200, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	r->drawString("ARROW KEYS!", wid/2, logoY + 240, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.5f);

	int rooty = logoY + 340; 
	
	
	dg->drawButton(160-1, rooty-6, 120);
	r->drawString("Menu", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	

	
}  

void InstructionsState::keyPressed(unsigned int key) {

}
void InstructionsState::keyReleased(unsigned int key) {

}
void InstructionsState::mouseMoved(int x, int y, int oldx, int oldy) {

}
 
InstructionsState::~InstructionsState() {

}

