/*
 * MenuState.cpp
 *
 *  Created on: 08 Jun 2011
 *      Author: Ashley
 */

#include "MenuState.h"
#include "../DefaultGame.h"

#include "../objects/Player.h"

MenuState::MenuState(): GameState() {

}

unsigned int MenuState::id() {
	return StatesList::STATE_MENU;
}

void MenuState::enter(GameContainer* container, StateBasedGame* game, GameState* from) { 
	m_introTimer = 0.001f; 
	m_outroTimer = 0.0f;
}
void MenuState::leave(GameContainer* container, StateBasedGame* game, GameState* to) { }

void MenuState::init(GameContainer* container, StateBasedGame* game) {

	DefaultGame* dg = DefaultGame::getInstance();
	m_selectedIndex = 0;

	m_introTimer = 0.0001f;
	m_introDuration = 0.8f;

	m_outroTimer = 0.0f;
	m_outroDuration = 0.8f;

	new Player(); // s_init
	player = Player::s_spr->getCopy();
	player->setCenterOfRotation(player->getWidth()/2, player->getHeight()/2);

	//logo = dg->spritesheet->getSubImage(dg->getItemByName("logo.png"))->

}

void MenuState::update(GameContainer* container, StateBasedGame* game, GameTimer* timer) {
	DefaultGame* dg = DefaultGame::getInstance();
	Input* i = ARK2D::getInput();

	if (m_introTimer > 0.0f) {
		m_introTimer += timer->getDelta();
		if (m_introTimer >= m_introDuration) {
			m_introTimer = 0.0f;
		}
	}
	
	if (m_outroTimer > 0.0f) {
		m_outroTimer += timer->getDelta();
		if (m_outroTimer >= m_outroDuration) {
			dg->enterState(dg->stateInstructions);
		}
	}

	if (i->isKeyPressed(Input::KEY_W) || i->isKeyPressed(Input::KEY_UP) || i->isGamepadButtonPressed(Gamepad::DPAD_UP)) {
		dg->m_sfxMenuSwitch->play();
		m_selectedIndex--; 
		if (m_selectedIndex < MENU_BEGIN) {
			m_selectedIndex = MENU_END; 
		}
	}
	if (i->isKeyPressed(Input::KEY_S) || i->isKeyPressed(Input::KEY_DOWN) || i->isGamepadButtonPressed(Gamepad::DPAD_DOWN)) {
		dg->m_sfxMenuSwitch->play();
		m_selectedIndex++;
		if (m_selectedIndex > MENU_END) {
			m_selectedIndex = MENU_BEGIN;
		}
	}

	player->rotate(180.0f * timer->getDelta());
 
	if (i->isKeyPressed(Input::KEY_ENTER) || i->isGamepadButtonPressed(Gamepad::BUTTON_START) || i->isGamepadButtonPressed(Gamepad::BUTTON_A)) {
		//dg->sfx_ui_select->play();
		if (m_selectedIndex == MENU_PLAY) {
			//DefaultGame::getInstance()->m_sfxStart->play(); 
			dg->m_sfxMenuConfirm->play();

			dg->stateInGame->reset();
			dg->stateInGame->start(); 
			game->enterState(dg->stateInGame);

		} else if (m_selectedIndex == MENU_INSTRUCTIONS) {
			
			if (m_outroTimer == 0.0f) {
				dg->m_sfxMenuConfirm->play();
				m_outroTimer = 0.001f;
			}
			//game->enterState(dg->stateInstructions);
		} 
		else if (m_selectedIndex == MENU_EXIT) {
			//dg->sfx_ui_select->stop();
			container->close();
		}
	} 
}

void MenuState::render(GameContainer* container, StateBasedGame* game, Renderer* r) {
	DefaultGame* dg = DefaultGame::getInstance();
	dg->backdrop->render(container, r);

	InGameState* igs = dg->stateInGame;
	//r->drawString("Menu", 20, 20, Renderer::ALIGN_LEFT, Renderer::ALIGN_TOP);

	float logoY = 100;
	if (m_introTimer > 0.0f) {
		logoY = Easing::ease(Easing::QUADRATIC_OUT, m_introTimer, 505, -405.0f, m_introDuration);
	}else if (m_outroTimer > 0.0f) {
		logoY = Easing::ease(Easing::QUADRATIC_OUT, m_outroTimer, 100, -480.0f, m_outroDuration);
	}

	r->drawString("SINKING", 160, logoY, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 3.0f);
	r->drawString("FEELING", 160, logoY + 40, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 3.0f);
 
	r->setDrawColor(Color::white);


	player->setColor(dg->backdrop->getTint());
	player->drawCentered(160.0f, logoY+128);
	
	int rooty = logoY + 220; //320;
	int wid = (signed int) container->getWidth();

	
	if (m_selectedIndex == MENU_PLAY) {
		dg->drawButton(160-2, rooty-6, 100);
		r->drawString("Play", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	} else {
		r->drawString("Play", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
	}

	rooty += 40;
	if (m_selectedIndex == MENU_INSTRUCTIONS) {

		dg->drawButton(160-2, rooty-6, 240);
		r->drawString("Instructions", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	} else {
		r->drawString("Instructions", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
	}
  
	// if (m_selectedIndex == MENU_CREDITS) {
	// 	r->getFont()->drawStringCenteredAt("< Credits >", wid/2, rooty+60);
	// } else {
	// 	r->getFont()->drawStringCenteredAt("Credits", wid/2, rooty+60);
	// }

	rooty += 40;
	if (m_selectedIndex == MENU_EXIT) {
		dg->drawButton(160-1, rooty-6, 100);
		r->drawString("Exit", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	} else {
		r->drawString("Exit", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
	}
 
 	
}

MenuState::~MenuState() {

}