/*
 * SummaryState.cpp
 *
 *  Created on: 08 Jun 2011
 *      Author: Ashley
 */

#include "SummaryState.h"
#include "../DefaultGame.h"
#include "../objects/Player.h"

SummaryState::SummaryState(): GameState() {

}

unsigned int SummaryState::id() {
	return StatesList::STATE_SUMMARY;
}

void SummaryState::enter(GameContainer* container, StateBasedGame* game, GameState* from) { 
	m_introTimer = 0.001f;
	m_outroTimer = 0.0f;
}
void SummaryState::leave(GameContainer* container, StateBasedGame* game, GameState* to) { 

}

void SummaryState::init(GameContainer* container, StateBasedGame* game) {
	DefaultGame* dg = DefaultGame::getInstance();

	m_introTimer = 0.001f;
	m_introDuration = 1.0f;

	m_outroTimer = 0.0f;
	m_outroDuration = 0.8f;

	m_selectedIndex = MENU_RETRY;
}

void SummaryState::update(GameContainer* container, StateBasedGame* game, GameTimer* timer) {
	
	DefaultGame* dg = DefaultGame::getInstance();
	Input* in = ARK2D::getInput();

	dg->stateInGame->update(container, game, timer);	

	if (m_introTimer > 0.0f) {
		m_introTimer += timer->getDelta();
		if (m_introTimer >= m_introDuration) {
			m_introTimer = 0.0f;
		}
	}

	if (m_outroTimer > 0.0f) {
		m_outroTimer += timer->getDelta();
		if (m_outroTimer >= m_outroDuration) {
			dg->enterState(dg->stateMenu);
		}
	}

	if (in->isKeyPressed(Input::KEY_W) || in->isKeyPressed(Input::KEY_UP) || in->isGamepadButtonPressed(Gamepad::DPAD_UP)) {
		dg->m_sfxMenuSwitch->play();
		m_selectedIndex--; 
		if (m_selectedIndex < MENU_BEGIN) {
			m_selectedIndex = MENU_END; 
		}
	}
	if (in->isKeyPressed(Input::KEY_S) || in->isKeyPressed(Input::KEY_DOWN) || in->isGamepadButtonPressed(Gamepad::DPAD_DOWN)) {
		dg->m_sfxMenuSwitch->play();
		m_selectedIndex++;
		if (m_selectedIndex > MENU_END) {
			m_selectedIndex = MENU_BEGIN;
		}
	}

	if (in->isKeyPressed(Input::KEY_ENTER) || in->isGamepadButtonPressed(Gamepad::BUTTON_START) || in->isGamepadButtonPressed(Gamepad::BUTTON_A)) {
		if (m_selectedIndex == MENU_RETRY) {
			dg->m_sfxMenuConfirm->play();
			dg->stateInGame->reset();
			dg->stateInGame->start(); 
			game->enterState(dg->stateInGame);

		} else if (m_selectedIndex == MENU_EXIT) {
			if (m_introTimer == 0.0f && m_outroTimer == 0.0f) { 
				m_outroTimer = 0.001f;
				dg->m_sfxMenuBack->play();
			}
		}
	}
}  

void SummaryState::render(GameContainer* container, StateBasedGame* game, Renderer* r) {
	DefaultGame* dg = DefaultGame::getInstance();
	dg->stateInGame->render(container, game, r);	

	float logoY = 95; //95;
	if (m_introTimer > 0.0f) {
		logoY = Easing::ease(Easing::QUADRATIC_OUT, m_introTimer, 505, -410.0f, m_introDuration); // 410
	} 
	if (m_outroTimer > 0.0f) {
		logoY = Easing::ease(Easing::QUADRATIC_OUT, m_outroTimer, 95, -480.0f, m_outroDuration); // 95
	}



	r->drawString("GAME", 160, logoY-5, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 3.0f);
	r->drawString("OVER", 160, logoY+35, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 3.0f);

	// score
	r->drawString(Cast::toString<unsigned int>(dg->stateInGame->m_score), 160, logoY + 145, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 4.0f);
	r->drawString(StringUtil::append("BEST: ", dg->stateInGame->m_scoreHigh), 160, logoY + 70, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
 
	int rooty = logoY + 245; //240;
	int wid = (signed int) container->getWidth(); 
	
	if (m_selectedIndex == MENU_RETRY) {
		dg->drawButton(160-1, rooty-6, 120);
		r->drawString("Retry", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	} else {
		r->drawString("Retry", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
	}

	rooty += 40;
	if (m_selectedIndex == MENU_EXIT) {
		dg->drawButton(160-1, rooty-6, 100);
		r->drawString("Menu", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
	} else {
		r->drawString("Menu", wid/2, rooty, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
	}

	//r->drawString(StringUtil::appendf("f: ", m_outroTimer), wid/2, 440, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 1.0f);
}

SummaryState::~SummaryState() {

}

